@@ -1,3 +1,16 @@
+/* A utility program originally written for the Linux OS SCSI subsystem.
+ *  Copyright (C) 1999-2013 D. Gilbert
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2, or (at your option)
+ *  any later version.
+ *
+ * This program uses the SCSI command READ BUFFER on the given
+ * device, first to find out how big it is and then to read that
+ * buffer (data mode, buffer id 0).
+ */
+
+
 #define _XOPEN_SOURCE 500
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE
@@ -24,19 +37,6 @@
 #include "sg_lib.h"
 #include "sg_io_linux.h"
 
-/* A utility program originally written for the Linux OS SCSI subsystem.
- *  Copyright (C) 1999-2009 D. Gilbert
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
- *
- * This program uses the SCSI command READ BUFFER on the given
- * device, first to find out how big it is and then to read that
- * buffer (data mode, buffer id 0).
- */
-
-
 #define RB_MODE_DESC 3
 #define RB_MODE_DATA 2
 #define RB_DESC_LEN 4
@@ -51,7 +51,7 @@
 #endif
 
 
-static char * version_str = "4.88 20090908";
+static const char * version_str = "4.90 20131014";
 
 static struct option long_options[] = {
         {"buffer", 1, 0, 'b'},
@@ -82,7 +82,9 @@ struct opts_t {
     int opt_new;
 };
 
-static void usage()
+
+static void
+usage()
 {
     fprintf(stderr, "Usage: sg_rbuf [--buffer=EACH] [--dio] [--help] "
             "[--mmap] [--quick]\n"
@@ -104,7 +106,8 @@ static void usage()
             "repeatedly\n");
 }
 
-static void usage_old()
+static void
+usage_old()
 {
     printf("Usage: sg_rbuf [-b=EACH_KIB] [-d] [-m] [-q] [-s=OVERALL_MIB] "
            "[-t] [-v] [-V]\n               DEVICE\n");
@@ -123,7 +126,8 @@ static void usage_old()
            "repeatedly\n");
 }
 
-static void usage_for(const struct opts_t * optsp)
+static void
+usage_for(const struct opts_t * optsp)
 {
     if (optsp->opt_new)
         usage();
@@ -131,7 +135,8 @@ static void usage_for(const struct opts_t * optsp)
         usage_old();
 }
 
-static int process_cl_new(struct opts_t * optsp, int argc, char * argv[])
+static int
+process_cl_new(struct opts_t * optsp, int argc, char * argv[])
 {
     int c, n;
     int64_t nn;
@@ -214,7 +219,8 @@ static int process_cl_new(struct opts_t * optsp, int argc, char * argv[])
     return 0;
 }
 
-static int process_cl_old(struct opts_t * optsp, int argc, char * argv[])
+static int
+process_cl_old(struct opts_t * optsp, int argc, char * argv[])
 {
     int k, jmp_out, plen, num;
     int64_t nn;
@@ -301,7 +307,8 @@ static int process_cl_old(struct opts_t * optsp, int argc, char * argv[])
     return 0;
 }
 
-static int process_cl(struct opts_t * optsp, int argc, char * argv[])
+static int
+process_cl(struct opts_t * optsp, int argc, char * argv[])
 {
     int res;
     char * cp;
@@ -322,7 +329,8 @@ static int process_cl(struct opts_t * optsp, int argc, char * argv[])
 }
 
 
-int main(int argc, char * argv[])
+int
+main(int argc, char * argv[])
 {
     int sg_fd, res, j;
     unsigned int k, num;
@@ -333,7 +341,7 @@ int main(int argc, char * argv[])
     int buf_capacity = 0;
     int buf_size = 0;
     int64_t total_size = RB_DEF_SIZE;
-    size_t psz = getpagesize();
+    size_t psz;
     int dio_incomplete = 0;
     struct sg_io_hdr io_hdr;
     struct timeval start_tm, end_tm;
@@ -342,6 +350,11 @@ int main(int argc, char * argv[])
 #endif
     struct opts_t opts;
 
+#if defined(HAVE_SYSCONF) && defined(_SC_PAGESIZE)
+    psz = sysconf(_SC_PAGESIZE); /* POSIX.1 (was getpagesize()) */
+#else
+    psz = 4096;     /* give up, pick likely figure */
+#endif
     memset(&opts, 0, sizeof(opts));
     res = process_cl(&opts, argc, argv);
     if (res)
@@ -478,6 +491,7 @@ int main(int argc, char * argv[])
             printf("out of memory (data)\n");
             return SG_LIB_CAT_OTHER;
         }
+        /* perhaps use posix_memalign() instead */
         if (opts.do_dio)    /* align to page boundary */
             rbBuff= (unsigned char *)(((unsigned long)rawp + psz - 1) &
                                       (~(psz - 1)));
@@ -600,8 +614,8 @@ int main(int argc, char * argv[])
     }
     if (dio_incomplete)
         printf(">> direct IO requested but not done\n");
-    printf("Read %"PRId64" MiB (actual: %"PRId64" bytes), buffer size=%d KiB "
-           "(%d bytes)\n", (total_size / (1024 * 1024)),
+    printf("Read %" PRId64 " MiB (actual: %" PRId64 " bytes), buffer "
+           "size=%d KiB (%d bytes)\n", (total_size / (1024 * 1024)),
            (int64_t)num * buf_size, buf_size / 1024, buf_size);
 
     if (rawp) free(rawp);
