@@ -1,3 +1,14 @@
+/* A utility program originally written for the Linux OS SCSI subsystem
+*  Copyright (C) 2003-2014 D. Gilbert
+*  This program is free software; you can redistribute it and/or modify
+*  it under the terms of the GNU General Public License as published by
+*  the Free Software Foundation; either version 2, or (at your option)
+*  any later version.
+
+   This program issues the SCSI SEND DIAGNOSTIC command and in one case
+   the SCSI RECEIVE DIAGNOSTIC command to list supported diagnostic pages.
+*/
+
 #include <unistd.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -13,18 +24,7 @@
 #include "sg_cmds_basic.h"
 #include "sg_cmds_extra.h"
 
-/* A utility program originally written for the Linux OS SCSI subsystem
-*  Copyright (C) 2003-2009 D. Gilbert
-*  This program is free software; you can redistribute it and/or modify
-*  it under the terms of the GNU General Public License as published by
-*  the Free Software Foundation; either version 2, or (at your option)
-*  any later version.
-
-   This program issues the SCSI SEND DIAGNOSTIC command and in one case
-   the SCSI RECEIVE DIAGNOSTIC command to list supported diagnostic pages.
-*/
-
-static char * version_str = "0.36 20090617";
+static const char * version_str = "0.40 20140110";
 
 #define ME "sg_senddiag: "
 
@@ -408,8 +408,11 @@ static int build_diag_page(const char * inp, unsigned char * mp_arr,
         *mp_arr_len = 0;
     if ('-' == inp[0]) {        /* read from stdin */
         char line[512];
+        char carry_over[4];
         int off = 0;
+        int split_line;
 
+        carry_over[0] = 0;
         for (j = 0; j < 512; ++j) {
             if (NULL == fgets(line, sizeof(line), stdin))
                 break;
@@ -418,11 +421,32 @@ static int build_diag_page(const char * inp, unsigned char * mp_arr,
                 if ('\n' == line[in_len - 1]) {
                     --in_len;
                     line[in_len] = '\0';
-                }
+                    split_line = 0;
+                } else
+                    split_line = 1;
             }
-            if (0 == in_len)
+            if (in_len < 1) {
+                carry_over[0] = 0;
                 continue;
-            lcp = line;
+            }
+            if (carry_over[0]) {
+                if (isxdigit(line[0])) {
+                    carry_over[1] = line[0];
+                    carry_over[2] = '\0';
+                    if (1 == sscanf(carry_over, "%x", &h))
+                        mp_arr[off - 1] = h;       /* back up and overwrite */
+                    else {
+                        fprintf(stderr, "build_diag_page: carry_over error "
+                                "['%s'] around line %d\n", carry_over, j + 1);
+                        return 1;
+                    }
+                    lcp = line + 1;
+                    --in_len;
+                } else
+                    lcp = line;
+                carry_over[0] = 0;
+            } else
+                lcp = line;
             m = strspn(lcp, " \t");
             if (m == in_len)
                 continue;
@@ -444,6 +468,10 @@ static int build_diag_page(const char * inp, unsigned char * mp_arr,
                                 j + 1, (int)(lcp - line + 1));
                         return 1;
                     }
+                    if (split_line && (1 == strlen(lcp))) {
+                        /* single trailing hex digit might be a split pair */
+                        carry_over[0] = *lcp;
+                    }
                     if ((off + k) >= max_arr_len) {
                         fprintf(stderr, "build_diag_page: array length "
                                 "exceeded\n");
@@ -484,8 +512,8 @@ static int build_diag_page(const char * inp, unsigned char * mp_arr,
                     return 1;
                 }
                 mp_arr[k] = h;
-                cp = strchr(lcp, ',');
-                c2p = strchr(lcp, ' ');
+                cp = (char *)strchr(lcp, ',');
+                c2p = (char *)strchr(lcp, ' ');
                 if (NULL == cp)
                     cp = c2p;
                 if (NULL == cp)
@@ -533,6 +561,7 @@ static struct page_code_desc pc_desc_arr[] = {
         {0x3f, "Protocol specific (SAS transport)"},
         {0x40, "Translate address (direct access)"},
         {0x41, "Device status (direct access)"},
+        {0x42, "Rebuild assist (direct access)"}, /* sbc3r31 */
 };
 
 static const char * find_page_code_desc(int page_num)
@@ -591,6 +620,10 @@ int main(int argc, char * argv[])
     }
 
     if (NULL == opts.device_name) {
+        if (opts.do_list) {
+            list_page_codes();
+            return 0;
+        }
         fprintf(stderr, "No DEVICE argument given\n");
         if (opts.opt_new)
             usage();
@@ -655,18 +688,6 @@ int main(int argc, char * argv[])
                        "'-raw='\n");
         }
     }
-    if (NULL == opts.device_name) {
-        if (opts.do_list) {
-            list_page_codes();
-            return 0;
-        }
-        fprintf(stderr, "No DEVICE argument given\n");
-        if (opts.opt_new)
-            usage();
-        else
-            usage_old();
-        return SG_LIB_SYNTAX_ERROR;
-    }
 
     if ((sg_fd = sg_cmds_open_device(opts.device_name, 0 /* rw */,
                                      opts.do_verbose)) < 0) {
